/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.lock;
import org.jboss.logging.Logger;

/**
 * Factory to create LockStragtegy instance.
 *
 * @author Ben Wang
 */
public class LockStrategyFactory
{
   private static LockStrategyFactory s_me = null;
   private Logger log_ = Logger.getLogger(LockStrategyFactory.class);


   /**
    * Transaction locking isolation level. Default.
    */
   private static IsolationLevel lockingLevel_ = IsolationLevel.REPEATABLE_READ;

   /**
    *
    */
   protected LockStrategyFactory()
   {
   }

   public static LockStrategyFactory getInstance()
   {
      if (s_me == null)
         s_me = new LockStrategyFactory();

      return s_me;
   }

   public LockStrategy getLockStrategy()
   {
      log_.debug("Isolation level is: " +lockingLevel_);
      if (lockingLevel_ == null || lockingLevel_ == IsolationLevel.NONE)
         return new LockStrategyNone();
      if (lockingLevel_ == IsolationLevel.REPEATABLE_READ)
         return new LockStrategyRepeatableRead();
      if (lockingLevel_ == IsolationLevel.SERIALIZABLE)
         return new LockStrategySerializable();
      if (lockingLevel_ == IsolationLevel.READ_COMMITTED)
         return new LockStrategyReadCommitted();
      if (lockingLevel_ == IsolationLevel.READ_UNCOMMITTED)
         return new LockStrategyReadUncommitted();
      throw new RuntimeException("getLockStrategy: LockStrategy selection not recognized." +
            " selection: " + lockingLevel_);
   }

   public static void setIsolationLevel(IsolationLevel level)
   {
      lockingLevel_ = level;
   }

}
